%% SEG4KING_S8.02

%{
Prepare or modify SEG-Y for automatic loading in KINGDOM-SUITE or other programs 
 geological interpretation software and some more utilities.
AUTHOR: Marcel.li Farran (ICM-CSIC), 19 / I / 2018

New in version 8
 - "Group coordinates" (-9) as the origin of the coordinates.
- Repositioning of towed sources, the original position becomes cdpX & Y.
- Replaces  the position od gps antena by  Mid-point between source and detector 
   making easy CDP creation,sorting and staking.
- Repositioning for towed sources, original goes to cdpX & Y.
 
From previous versions
- Change coordinate format to UTM. (WGS84)
- Saves in SEG-Y compatible Kingdom-Suite with long. Of the constant trace.
- Allows change of UTM spindle.
- Select directory and files with the corresponding wildcard
- Interpolate if the coordinates are equal for lack of precision in the
  Decimals (type 2).
- Resampling of trace samples.
- Allows filtering of spikes (interference) by entering a value of
reference. It is considered spike if for a time t the value in the trace
Is larger than the adjacent ones multiplied by the reference value.
Con Matlab Signal Proc. Toolbox uses RMS with 999.
- Filter "stopped boat" or almost by works in station.
- Amplitude Envelope Instance (Matlab Signal Procesing Toolbox required)


###########################################################################
# - To the name of the resulting file is added a letter only if the data
#  are modifier respecting the original and are stored in a subfolder
# F = spikes filtered, r = resampled, e = envelope, s = nav. Smoothed,
# T = applied layback, or  multichannel geometry applied, M = merged files
# Name_S# = Splitted file
# The order of processing of the trace in case everything is activated is
# 1 filtered, 2 envelope, 3 resampled, positions calculations( coordinates,
smoothing,offsets).  
###########################################################################

Previous to S6 version it was an error  in the UTM zone changer.
These errors can now be solved using the format Coordinate input -9.

%}
%{
 SEGYMAT == Valid Values  ===============
 REV| DSF|Data Sample Format	          |
 0  |  1 | 4 Byte IBM Floating Point	  |        
 1  |  1 | 4 Byte IBM Floating Point	  | >>>> Kingdom S. Default Input
 1  |  2 | 4 Byte two's complement Integer|	
 1  |  3 | 2 Byte two's complement Integer|	
 1  |  5 | 4 Byte IEEE FLoating Point	  | >>>> ATLAS, TOPAS Parametric
===========================================
%}
%{
Traces can be converted fron Real Signal to Instaneous Amplitud Envelope
 useful for a better visualization on Kingdom Suite for
 very high frequency data as SIMRAD Topas and other Parametric
 echosounders. In most of cases data from these echosounders are recorded
 as Inst. Ampl. Env. mode yet. 
MATLAB SIGNAL PROCESSING TOOLBOX Required for this function.
 
 If MATLAB MAPPING TOOLBOX exist a simple map of lines is created. 
 also KML file is created with segy files metada. 
 CREATED FILES will have the same name but in a new KS_OK subfolder  
%}                   
%% INICI

    clc;clear all;echo off;
    SEG4KING_Version = 'Version: 8.0 SEG4KING (I/2018)';
    tit=SEG4KING_Version;
    color=('g'); 
    warning ('off','all');
    ver;

    % test if segymat exists
    if exist('SegymatVersion','file')== 0 
        disp 'SEGYMAT not found, Install or re-start Matlab ';
        pathtool;
        break;
    else 
        SegymatVersion;
    end

    Current_Folder = uigetdir(pwd ,'Directory of SEG-Y data files');
    cd(Current_Folder);
    total=0;ccc=1;
    if exist('S4K8def.mat','file') == 2
        d_i = load ('S4K8def.mat');
        defans = d_i.answer;    
    else
        defans = {'3';'*.sgy';'1';'30';'30';'N';'0';'1';'1';'N';'0';'0';'N';'0';'my survey'};
    end

%==========================================================================
   
    ps1=     ' INPUT FORMAT:  IEEE: 5   IBM4Byte: 1   IBM2Byte: 3       [SEG4KING Output is format 1]' ;
    ps2=     ' EXTENSION: *(.seg, .sgy, .segy, .tra, ...)                                              ';

    ps3= [   ' COORDINATES INPUT FORMAT (WGS84) in SourceX&Y Fields :                                  '...
             ' ... Standards:   UTM(m): 0    Degree/Miliseconds : 1                                    '...
             ' ... Other:     UTM(cm): -2    UTM(dm): -1       UTM(m. in GroupX&Y fields): -9           '... 
             ' ... DDMMM(smoothed): 2       DDDMMMMM: 3           Degree/Centisecond: 4                ' ];

    ps4=     ' INPUT  UTM ZONE (For UTM input or  Fuse change';

    ps5=     ' OUTPUT UTM ZONE (Applied to all files)';

    ps6=     ' HEMISPHERE [N/S]';

    ps7=  [  ' FILTER "ON STATION" files: Minimum Mean speed accepted (in Knots).                      '...
             '    These Files will be isolated to special folder                                       '];

    ps8= [   ' SIGNAL SPIKES FILTER [integers] 0 or 1: No Filter, 2-Any value: Filter ON, 999:AUTO     '...
             ' ... X > 1 YES    If absolute Amplitude is >  X times the neighbours traces Amplitudes,  '...
             ' ... then is considered an SPIKE and the value is replaced by mean value of neighbours,  '...
             ' ... Auto ( 999 ) ******** MATLAB Sig.Proc.Toolbox necessary *******                     '];

    ps9=     ' RESAMPLING FACTOR   [1 = no resampling,  2 = trace_samples/2 ...]                       ';

    ps10=    [' CONVERT Real Signal to Inst. Ampl. Envelope [Y/N]                                        '...
              '*** MATLAB Sig.Proc.Toolbox necessary ***'          ];

    ps11=    [' SMOOTH NAVIGATION [integer] - Number of values Before & After to Average each position '...
              '... ( p.e: 3 means an mov. aver. of 7 values)           '];

    ps12=[    ' OFFSETS IN METERS / DEPH POINT Pos. [integ.] (Shalow water towed sources or MCH)       '...
              ' .. One value (or two equal): then X & Y are recalculated in XY and original XY are     '...
              ' ... stored in groupX-Y fields                                                          '...
              ' .. Many Values: (ex: 25 30 60 90 )  interpreted as GPS antena to seismic source and    '...
              ' ...succesive channels distances, 3 in the exemple  FieldRecord & TraceNumber trace     '...
              '... headers necessary, offsets written in byte 37-40 offset.TraceHeader                 '...
              '... DEPH-POINT calculated replace the sourceX&Y trace header, original moved to GroupX&Y'];

    ps13=     ' MERGE ALL processed files (Y/N) in Alphabetic order (Time for Gap split must be 0)     ';

    ps14=     ' Max. Time Gap accepted (Minutes integer): Bigger gaps SPLIT SegY file [0:NO split files]';
    ps15=     ' INPUT SURVEY NAME: ';
    prompt = {ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8,ps9,ps10,ps11,ps12,ps13,ps14,ps15};
    lins =[1,90];
    answer =inputdlg(prompt,tit,lins,defans);
      
    assignin('base','DSF_In',answer(1));DSF_In=str2num(char(DSF_In));REV_In=1 ;
    assignin('base','ext',answer(2)); ext=char(ext);
    assignin('base','COORDENADAS',answer(3)); COORDENADAS=str2num(char(COORDENADAS)); ORICOOR =COORDENADAS;
    assignin ('base','fromZ',answer{4});zoneF = double(str2num(fromZ));
    assignin('base','toZ',answer{5});zoneT =double(str2num(toZ));
    assignin('base','hemis',answer{6});
    assignin('base','minex',answer(7));minex = str2num(char(minex)); 
    assignin('base','DSPK',answer(8));DSPK=str2num(char(DSPK));
    assignin('base','RESAMPLE_FACTOR',answer(9));RESAMPLE_FACTOR = str2num(char(RESAMPLE_FACTOR));
    assignin('base','instamp',answer{10});instamp= upper(instamp);if instamp == 'Y'||instamp == 'y'; instamp = 'Y';end;
    assignin('base','smooth',answer(11)); smooth=str2num(char(smooth));
    assignin('base','towed',answer(12));  towed=str2num(char(towed));
    assignin('base','merged',answer(13)); mer=char(merged);if mer == 'Y'|| mer == 'y' ; mer = 'Y';end;
    assignin('base','maxGapT',answer(14)); GAP_T =str2num(char(maxGapT));if mer =='Y';GAP_T=0;end;
    assignin('base','survey',answer(15)); survey=char(survey);
    REV_Out=1; DSF_Out=1; % Output segy format
    savefile='S4K8def'; save(savefile, 'answer'); % Save defaults
    %_______________________________________
    files=dir(ext);fin=size (files,1);
    mkdir('./KS_OK'); mkdir('./Unknown_errors');mkdir('./Position_errors');
    mkdir('./stemp');mkdir('./OnStation_files');
    kmlName=['./KS_OK/' survey '.kml'];
    kmlfile = kmlName ; fid9 = fopen(kmlfile, 'w');  % Inicializa kml
    fprintf(fid9,'%s\r\n','<?xml version="1.0" encoding="UTF-8"?>');
    fprintf(fid9,'%s\r\n','<kml xmlns="http://earth.google.com/kml/2.1">');
    fprintf(fid9,'%s\r\n','<Document>');
    
    
    if mer == 'Y', mkdir('./KS_OK/Merged/');end;
    pwd;
    if fin == 0; 
        disp ' NO FILES WITH THIS EXTENSION IN THE SELECTED FOLDER';  break;
    end;  
    disp('NEW CREATED FILES will have the same name in a new subfolder : KS_OK ')

%% Splitting
if GAP_T > 0
    for k=1:fin     

        infile=files(k).name; outfile=infile; [pathstr, name, ex] = fileparts(outfile); 
        [data,SegyTraceHeaders,SegyHeader]=ReadSegy(infile,'revision',REV_In,'dsf',DSF_In);
        tr=size(data,2); 
        dia=[SegyTraceHeaders.DayOfYear];hora=[SegyTraceHeaders.HourOfDay];mins=[SegyTraceHeaders.MinuteOfHour];
        ttm=(dia*24*60)+(hora*60)+mins;  temps(1)=0;
        for i= 2:tr % calc. T and D partials
                   temps(i)=abs(ttm(i)-ttm(i-1));    
        end          
        tzini=1;tzfin=1;sgm=1;
        for j=2:tr     % find GAPS
            if temps(j) >= GAP_T
               tzfin=j-1;     Newdata = data(:,tzini:tzfin);
               NewSegyTraceHeaders = SegyTraceHeaders(tzini:tzfin);
               NewSegyHeader=SegyHeader;   trozo = int2str(sgm);
               NewName=['./stemp/' name '_S' trozo ex];
               WriteSegyStructure(NewName,NewSegyHeader,NewSegyTraceHeaders,Newdata,'revision',REV_In,'dsf',DSF_In);   
               tzini=j; sgm=sgm+1;
            else tzfin=j; trozo = int2str(sgm);
           end;
           if j==tr   
                Newdata = data(:,tzini:tzfin);
                NewSegyTraceHeaders = SegyTraceHeaders(tzini:tzfin);
                NewSegyHeader=SegyHeader;   trozo = int2str(sgm);
                if sgm== 1; trozo = '0';end; 
                NewName=[pathstr './stemp/' name '_S' trozo ex1];
                WriteSegyStructure(NewName,NewSegyHeader,NewSegyTraceHeaders,Newdata,'revision',REV_In,'dsf',DSF_In);   
           end;
        end; 
        clear data; 
        cartel = [name '------------> SEGMENTS: ' int2str(sgm)];
        disp (cartel);
    end;
     como=['*_S' ext];    prev= './' ;
else como=['../' ext];    prev= '../';
end
chdir ('stemp');pwd;files=dir (como);fin=size (files,1);
figure

%% Main
for k=1:fin
     
    COORDENADAS=ORICOOR;
    infile2=files(k).name;  infile = [prev, infile2];
    noticia = '                                               Propcessing Line:  ';
    str = [noticia, infile ' = '];  disp ' ' ;disp (str); disp ' ';
    
    
    [data,SegyTraceHeaders,SegyHeader]=ReadSegy(infile,'revision',REV_In,'dsf',DSF_In);    
    tr=size(data,2);
    %Total Time for actual line  
    dia=[SegyTraceHeaders.DayOfYear];        hora=[SegyTraceHeaders.HourOfDay];
	mins=[SegyTraceHeaders.MinuteOfHour];    segs=[SegyTraceHeaders.SecondOfMinute];
    diaf=dia(tr);          diai=dia(1);
    if diaf < diai, diaf=diai+1; end;
    horaf=hora(tr);      horai=hora(1);
    minutosf=mins(tr);   minutosi=mins(1);
    segundosf=segs(tr);  segundosi=segs(1);
    tiempoi=(diai*24)+horai+(minutosi/60)+(segundosi/3600);
    tiempof=(diaf*24)+horaf+(minutosf/60)+(segundosf/3600);
    ttrans(k)=tiempof-tiempoi; 
     
 %%  spikes removal -----------------------------------------------------
    
    if exist('sgolayfilt','file')== 0   % NO Existe
        disp '______________ SIGNAL PROCESSING TOOLBOX NOT INSTALLED, Use other than 999 value for Filter ';
        disp '__________________________________________ NOT FILTERED ____________________________';
        DPSK=1;
    end 

    if DSPK > 1 ;
        disp ' '; disp (' cleaning spikes ...'); disp ' ';
        dato=data;  mdata=rms(data);  smdata = sgolayfilt(mdata,3,41);
        difsm=abs(smdata-mdata); 
        for cc=2:1:size(data,2)-1; % para cada columna
            if SegyTraceHeaders(cc).DelayRecordingTime == SegyTraceHeaders(cc-1).DelayRecordingTime && ...
                SegyTraceHeaders(cc).DelayRecordingTime == SegyTraceHeaders(cc+1).DelayRecordingTime;
                if DSPK==999                      % AUTO  
                    if difsm(cc)>= 0.04 
                           dato(:,cc) = 0.5 * ( data(:,cc-1) + data(:,cc+1) );
                    end
                else                               % MANUAL
                    for ff=1:1:size(data,1);  % para cada file         
                        if (data(ff, cc-1) >= 0 && data(ff, cc+1) >=  0) || (data(ff, cc-1) < 0 && data(ff, cc+1) < 0) 
                          %ambos positivos o ambos negativos 
                            if abs(data(ff, cc)) >= DSPK * abs(data(ff,cc-1)) && abs(data(ff,cc)) >= DSPK * abs(data(ff,cc+1)) 
                                dato(ff, cc) = 0.5 * (data(ff,cc-1) + data(ff,cc+1)); 
                            end; 
                            if DSPK * abs(data(ff, cc)) < abs(data(ff,cc-1)) && DSPK * abs(data(ff,cc)) < abs(data(ff,cc+1)) 
                                dato(ff, cc) = 0.5 * ( data(ff,cc-1) + data(ff,cc+1) );
                            end;                      
                        end;   
                    end; %for ff
                end %if else DSPK
            end; %if
        end; %for cc
        data=dato; clear dato;
    end; 
   
  %% Inst. Ampl. Envelop -------------------------------------------------
    A = exist('hilbert','file');
    if A == 2 && instamp == 'Y'; 
        
            [m,n] = size(data);  datenv=zeros(m,n);
            for tnum=1:1:n
                tracex=data(:,tnum)'; tracex=tracex/max(tracex);
                data(:,tnum)=20*log10(abs(hilbert(tracex+eps)));
            end;
             disp ' '; disp (' Inst. Ampl. Envelope applied...'); disp ' ';
    else
        if instamp =='Y';
        disp ' ';
        disp '_______________________________ Instantaneous amplitude Envelope not posible without Hilbert Funcion';
        disp '_______________________________ Probably you have not installed (or is not in path) "Matlab Signal Processing Toolbox"';
        end;
    end;
    
 %% Data Resampling  (always but ...)------------------------------------
    if RESAMPLE_FACTOR == 0, RESAMPLE_FACTOR = 1; end;
    if RESAMPLE_FACTOR >1, disp ' '; disp (' Resampling...'); disp ' ';end
    Rdata=data(1:RESAMPLE_FACTOR:end,:);
    SegyHeader.dt= floor(SegyHeader.dt*RESAMPLE_FACTOR); 
    SegyHeader.ns=size(Rdata,1);
    SegyHeader.dtOrig= SegyHeader.dt; SegyHeader.nsOrig=SegyHeader.ns;
    punt=0;
    
 %% Coordinates headers reformat ----------------------------------------   
   switch COORDENADAS
    case  3   % format DDDMMMMM 
        Dlon=fix([SegyTraceHeaders.SourceX]/100000);
        Dlat=fix([SegyTraceHeaders.SourceY]/100000);
        Mlon=(([SegyTraceHeaders.SourceX]/100000)-Dlon)*100;
        Mlat=(([SegyTraceHeaders.SourceY]/100000)-Dlat)*100;
        Lon = Dlon + (Mlon/60);
        Lat = Dlat + (Mlat/60);
    case  2  %format  DDMMM  ( to interpolate)
        Dlon=fix([SegyTraceHeaders.SourceX]/1000);
        Dlat=fix([SegyTraceHeaders.SourceY]/1000);
        Mlon=(([SegyTraceHeaders.SourceX]/1000)-Dlon)*100;
        Mlat=(([SegyTraceHeaders.SourceY]/1000)-Dlat)*100;
        Lon = Dlon + (Mlon/60);    
        Lat = Dlat + (Mlat/60);
        x=double(Lon);  
        y=double(Lat);
        cdpX=x;   
        cdpY=y; %( pre interpolated coordinates )
        % polynomial fit
        difx=abs(max(x)-min(x));
        dify=abs(max(y)-min(y));
        ctx=0;cty=0;
        for i=2 :tr ; 
           if x(i)  ~= x(i-1); ctx = ctx+1;end;
           if y(i)  ~= y(i-1); cty = cty+1;end;
        end
        if tr >= 20 
            if difx >= dify && ctx >= 10
                [coef,S]=polyfit(x,y,5);
                Lon=linspace(x(1),x(tr), tr);
                Lat=polyval(coef,Lon); 
            end
            if dify > difx && cty >= 10     
                [coef,S]=polyfit(y,x,4);  
                Lat=linspace(y(1),y(tr), tr);
                Lon=polyval(coef,Lat);
            end
            if difx==dify && difx == 0 || ctx <10 && cty < 10
                Lon=x ;Lat=y; punt =1; 
            end
        end
        ctx=0;cty=0;
    case 1  %Degrees*3600000 (miliseconds of arc) 
        Lon=[SegyTraceHeaders.SourceX]/3600000;    
        Lat=[SegyTraceHeaders.SourceY]/3600000;
    case 4  %Degrees*360000 (centiseconds of arc) 
        Lon=[SegyTraceHeaders.SourceX]/360000;    
        Lat=[SegyTraceHeaders.SourceY]/360000;    
    otherwise  % metric system
        x=[SegyTraceHeaders.SourceX]; y=[SegyTraceHeaders.SourceY];
        if COORDENADAS == -1; x=round(x/10); y=round(y/10);   end
        if COORDENADAS == -2; x=round(x/100); y=round(y/100); end   
        if COORDENADAS == -9; x=[SegyTraceHeaders.GroupX]; y=[SegyTraceHeaders.GroupY]; end
        Lon=x;  Lat=y;     %para el mapa
        if zoneF ~= zoneT  % change UTM fuse 
            disp ' '; disp (' Changing UTM fuse ...'); disp ' ';
            COORDENADAS=9; % 
            [ Lon, Lat ] = UTMGeo(x,y,zoneF,hemis );
        end 
   end %switch
 % discrimine some known errors on segy navigation data-------------------------

    nav_error = 0;
    LonMax= max(Lon);     LonMin=min(Lon);     LatMax=max(Lat);     LatMin=min(Lat);
    if  COORDENADAS < 1   %metric?
        if  LonMax==0 || LonMin==0 || LatMax==0 || LatMin==0;nav_error = 1;end 
    end
    if  COORDENADAS > 0   %geo?
        if LatMin<-90 || LatMax>90 || LonMin<-180 || LonMax>360; nav_error = 1;end
    end
    % Create MAP
    B=exist('geoshow','file');
    [path, nom, ex] = fileparts(files(k).name);
    kn =['\fontsize{8} ' nom];
    ccc=-ccc;
    if ccc==1,color='r' ; else color = 'g'; end
    if B > 0 
        if nav_error == 0 && punt == 0
            if Lon(1) < 360 
                gmap= geoshow (Lat, Lon);      %geoshow
            else
              [Lon, Lat] = UTMGeo(x,y,zoneF,hemis ); gmap= geoshow (Lat, Lon);
            end           
            text(Lon(1),Lat(1), kn); set(gmap ,'Color',color,'LineWidth',2);
            if k==fin; 
                grid on; title('Labels on line start)');    
                saveas(gcf,'../KS_OK/MAP.jpg','jpg');   
            end    
        end 
    end
    
    % if necessary trans GEO to UTM -----------------------------------
    x=zeros(tr,1); y=zeros(tr,1);  tr=length(Lat);
    if COORDENADAS ==2  % only for preinterpolated coordinates
       [x,y]= GeoUTM (cdpX,cdpY,zoneT);
       for i=1:length(x)
            SegyTraceHeaders(i).cdpX = round(x(i)); 
            SegyTraceHeaders(i).cdpY = round(y(i)); 
       end
    end
    if COORDENADAS >= 1  % all GEO?
        disp ' '; disp (' Lat-Lon to UTM'); disp ' ';
        [x,y]= GeoUTM (Lon,Lat,zoneT);
        for i=1: length(x) 
            if ORICOOR > -8
                SegyTraceHeaders(i).GroupX = SegyTraceHeaders(i).SourceX; 
                SegyTraceHeaders(i).GroupY = SegyTraceHeaders(i).SourceY; 
            end
            SegyTraceHeaders(i).SourceX=round(x(i));                  
            SegyTraceHeaders(i).SourceY=round(y(i)); 
           
        end; 
    end;     
   
        
 % Total distance & speed in line (in n.m.)-----------------------------
    x=[SegyTraceHeaders.SourceX];   y=[SegyTraceHeaders.SourceY];
    LonMax= x(tr);  LonMin =x(1);   LatMax= y(tr);   LatMin=y(1);
    distt(k)=sqrt(double(((LatMax-LatMin)^2)+((LonMax-LonMin)^2)))/1852; %nm
    MeanSpeed(k)=single(distt(k)/ttrans(k));    % in knots
    noticia = ' Mean Speed for line ) ';
    str = [noticia, infile ' = ', num2str(MeanSpeed(k)), ' knots'];
     disp ' ';disp (str); disp ' ';

 %% smoothing -----Moving average 1 + 2*smooth shots/pings ----------------------------

if smooth > 0 
                  disp ' '; disp ('Smoothing... '); disp ' ';
                  N=1 + (2*smooth);  
                  for i=1 :1:tr    % store pre-smooth in groupX&Y
                    SegyTraceHeaders(i).GroupX = SegyTraceHeaders(i).SourceX; 
                    SegyTraceHeaders(i).GroupY = SegyTraceHeaders(i).SourceY;
                  end                 
                  for i=smooth+1:1:tr-smooth  
                      j=i-smooth;g=i+smooth;
                      mmvx(1:N) = SegyTraceHeaders(j:g).GroupX ;
                      mmvy(1:N) = SegyTraceHeaders(j:g).GroupY ;
                      SegyTraceHeaders(i).SourceX = int32(sum(mmvx)/N);                     
                      SegyTraceHeaders(i).SourceY = int32(sum(mmvy)/N);
                 end;
end  
 
 %% position calculations for shallow water towed sources OR multichannel offsets--------------------
 
    if size(towed,2)==1
        if towed > 0;
          disp ' '; disp ('applying single channel offset... '); disp ' ';      
                for i=2:1:tr         
                        ITX(i)=SegyTraceHeaders(i).SourceX - SegyTraceHeaders(i-1).SourceX ;    %Incremet X
                        ITY(i)=SegyTraceHeaders(i).SourceY - SegyTraceHeaders(i-1).SourceY ;
                        ITXY(i)=round(sqrt (double(ITX(i)^2+ITY(i)^2))) ;                  % distance to previous ping/shot
                end
                ITX(1)=0; ITY(1)=0; ITXY(1)=0;      
                for i=1:1:tr       %store original postion in GroupX&Y
                        SegyTraceHeaders(i).GroupX = SegyTraceHeaders(i).SourceX; 
                        SegyTraceHeaders(i).GroupY = SegyTraceHeaders(i).SourceY;
                        SegyTraceHeaders(i).offset = towed;
                        g = i ;  SUMA = ITXY(i);
                        while g > 1
                            if SUMA >= towed, break ,end
                            g = g - 1; SUMA = SUMA + ITXY(g);
                        end
                        if g > 1
                            SegyTraceHeaders(i).SourceX = SegyTraceHeaders(g-1).GroupX;
                            SegyTraceHeaders(i).SourceY = SegyTraceHeaders(g-1).GroupY;
                            ofo(i) = i - g;
                        else
                            SegyTraceHeaders(1).SourceX = SegyTraceHeaders(1).GroupX;
                            SegyTraceHeaders(1).SourceY = SegyTraceHeaders(1).GroupY;   
                        end
                end
                for i=1:1:tr
                   if ofo(i)>0, g=i-1;break;  end  
                end
                CITX=sum(ITX(1:g));  CITY=sum(ITY(1:g));
                for i=1:1:g           
                        SegyTraceHeaders(i).SourceX  =   SegyTraceHeaders(i).SourceX - CITX;
                        SegyTraceHeaders(i).SourceY   =  SegyTraceHeaders(i).SourceY - CITY;
                end
        end
    else    % size >1 
        disp ' '; disp (' Applying Multichannel Offsets ... '); disp ' ';
        channels =size(towed,2)-1;
        for ch=1:1:channels
            ChOff(ch) = towed(1)+((towed(ch+1)-towed(1))/2); 
        end; 
        for i= 1:1:tr
            ch(i)  =  SegyTraceHeaders(i).TraceNumber;
            sh(i)  =  SegyTraceHeaders(i).FieldRecord;
            SegyTraceHeaders(i).offset  =  int32(ChOff(ch(i)));
            SegyTraceHeaders(i).GroupX = SegyTraceHeaders(i).SourceX; 
            SegyTraceHeaders(i).GroupY = SegyTraceHeaders(i).SourceY;
        end
        for i=channels+1:1:tr
            if sh(i) > sh(i-1) 
              ITX(i)= SegyTraceHeaders(i).SourceX - SegyTraceHeaders(i-1).SourceX ;
              ITY(i)= SegyTraceHeaders(i).SourceY - SegyTraceHeaders(i-1).SourceY ;
              ITXY(i)=round(sqrt(double(ITX(i)^2+ITY(i)^2))) ;
            else
              ITX(i)= ITX(i-1);  ITY(i)=ITY(i-1);  ITXY(i)=ITXY(i-1);
            end
        end
        for i=1:1:channels
           ITX(i)=ITX(channels+1); ITY(i)=ITY(channels+1);  ITXY(i)=ITXY(channels+1);
        end
        for i=1:1:tr
            itx(i) =  ChOff(ch(i))*  ITX(i)  / ITXY(i);
            ity(i) =  ChOff(ch(i))*  ITY(i) /  ITXY(i);
            itxy(i)=round(sqrt(double(itx(i)^2+ity(i)^2))) ;
            SegyTraceHeaders(i).SourceX  = SegyTraceHeaders(i).GroupX - itx(i);
            SegyTraceHeaders(i).SourceY  = SegyTraceHeaders(i).GroupY - ity(i);
        end
   end % size...  
     
 %  File sorting ----------------------------------------------
   last=size(SegyTraceHeaders,2);
   TSL = GetSegyTraceHeaderInfo(SegyTraceHeaders(1,last),'TraceSequenceLine');    
   if (TSL >= 10000000)||(TSL == 0)  
        outfile=['../Unknown_errors/' infile2];     % directorio erroneos
        
        disp ' ';disp (' Some unknow error in SEGY files, see corresponding folder');disp ' ';
   else 
      if nav_error==0 
          if MeanSpeed(k) <= minex && minex > 0 || LonMax==LonMin  && LatMin==LatMax 
            outfile=['../OnStation_Files/'  infile2]; % "too slow" file
            disp;disp (' Some "On Station" files, see On Station folder');disp;
          else   
            if RESAMPLE_FACTOR > 1, infile2=[ 'r' infile2]; end;
            if instamp=='Y', infile2= ['e' infile2]; end;
            if DSPK > 1, infile2= [ 'f' infile2]; end;
            if smooth > 0, infile2= [ 's' infile2]; end;
            if towed(1) > 0 , infile2= [ 't' infile2]; end;
            if size(towed) > 1 , infile2= [ 'o' infile2]; end;
            total = total + tr;
            outfile=['../KS_OK/' infile2];  % OK folder   
          end   
      else   % nav_error==1
          outfile=['../Position_errors/' infile2]; % errores de navegacion
          disp ' ';disp (' Some Position errors, see corresponding folder');disp ' ';
      end; 
   end; 
 
   %-----------Write output file (SEGY & KML) ---------
    fprintf(fid9,'<Placemark>\r\n');
    fprintf(fid9,strcat('<name>',files(k).name,'</name>\r\n'));
    fprintf(fid9,'<ExtendedData>\r\n');   
    fprintf(fid9,strcat('<Data name="Survey"><value>',survey,'</value></Data>\r\n'));
    fprintf(fid9,strcat('<Data name="Segyfile"><value>',files(k).name,'</value></Data>\r\n'));
    fprintf(fid9,'<Data name="Traces"><value>%7g</value></Data>\r\n',tr);

    fprintf(fid9,'<Data name="Samples/trace"><value>%10f</value></Data>\r\n',SegyHeader.ns);
    fprintf(fid9,'<Data name="Sample_Int."><value>%10f</value></Data>\r\n',SegyHeader.dt);
    fprintf(fid9,'<Data name="Lon-S"><value>%-+10.5f</value></Data>\r\n',Lon(1));
    fprintf(fid9,'<Data name="Lat-S"><value>%-+10.5f</value></Data>\r\n',Lat(1));
    fprintf(fid9,'<Data name="Lon-E"><value>%-+10.5f</value></Data>\r\n',Lon(tr));
    fprintf(fid9,'<Data name="Lat-E"><value>%-+10.5f</value></Data>\r\n',Lat(tr));
    fprintf(fid9,'<Data name="Zone_UTM"><value>%g</value></Data>\r\n',zoneT);
    fprintf(fid9,'<Data name="Speed (kn)"><value>%4.2f</value></Data>\r\n',MeanSpeed(k));
    fprintf(fid9,'<Data name="Time Start"><value>%2g:%2g:%2g</value></Data>\r\n',...
        SegyTraceHeaders(1).HourOfDay,SegyTraceHeaders(1).MinuteOfHour,SegyTraceHeaders(1).SecondOfMinute );    
    fprintf(fid9,'<Data name="Time End"><value>%2g:%2g:%2g</value></Data>\r\n',...
        SegyTraceHeaders(tr).HourOfDay,SegyTraceHeaders(tr).MinuteOfHour,SegyTraceHeaders(1).SecondOfMinute ); 
    fprintf(fid9,'<Data name="Date (Y/JD)"><value>%4g/%3g </value></Data>\r\n',...
        SegyTraceHeaders(1).YearDataRecorded, SegyTraceHeaders(1).DayOfYear);  
    fprintf(fid9,'</ExtendedData>\r\n');
    fprintf(fid9,'<Point><coordinates>%g,%g,0</coordinates></Point>\r\n',Lon(1),Lat(1));
    fprintf(fid9,'</Placemark>\r\n');
    fprintf(fid9,'<Placemark>\r\n');
    fprintf(fid9,'<LineString>\r\n');
    fprintf(fid9,'<coordinates>\r\n');
    for i= 1:1:tr
       fprintf(fid9,'%g,%g,0\r\n',Lon(i),Lat(i));
        SegyTraceHeaders(i).TraceSequenceFile=i;
        SegyTraceHeaders(i).CoordinateUnits=1;                 
        SegyTraceHeaders(i).SourceGroupScalar=1; 
        SegyTraceHeaders(i).LagTimeA = SegyTraceHeaders(i).DelayRecordingTime;
        SegyTraceHeaders(i).dt= SegyHeader.dt;                 
        SegyTraceHeaders(i).ns= SegyHeader.ns;                      
        tops(i)=SegyTraceHeaders(i).LagTimeA;
    end;
    fprintf(fid9,'</coordinates>\r\n');
    fprintf(fid9,'</LineString>\r\n');
    fprintf(fid9,'</Placemark>\r\n');
    orden(k)=k;
    ran(k)=int32(SegyHeader.dt * SegyHeader.ns / 1000);
    maxtop(k)=int32(max(tops)+ran(k));
    mintop(k)=int32(min(tops));  
    WriteSegyStructure(outfile,SegyHeader,SegyTraceHeaders,Rdata,'revision',REV_Out,'dsf',DSF_Out);
end;  % k=fin
fprintf(fid9,'%s\r\n','</Document>');
fprintf(fid9,'%s\r\n','</kml>');
%% ---------Remove empty folders ------------------------

fclose ('all');chdir ('..');rmdir('stemp','s')
if any(size(dir(['./Unknown_errors/' ext ]),1))  == 0, rmdir('./Unknown_errors'); end
if any(size(dir(['./OnStation_files/' ext ]),1)) == 0, rmdir('./OnStation_files');end
if any(size(dir(['./Position_errors/' ext ]),1)) == 0, rmdir('./Position_errors');end

%-------------Ranges display --------------------------

figure; ccc=1;
ploteo = errorbar(orden,mintop,(ran-ran),maxtop-mintop,':kv','LineWidth',2);
set(gca,'YDir','reverse');
grid off;
for i=1:1:max(orden)
    ccc=-ccc;
    if ccc==1,color='r' ; else color = 'g'; end
    [path, nom, ex] = fileparts(files(i).name); 
    ks=['\fontsize{10} ' nom]; nks = strrep(ks, '_', '\_');
    text(double(i)-0.2,double(mintop(i))+50,nks,'Rotation',270,'color',color);
end
ylabel('Minimum TimeDelay & Full Range in file','fontsize',11);
xlabel('Map file number','fontsize',8);
saveas(gcf,'./KS_OK/segy-ranges.jpg','jpg')



% -----------Merge created files ?--------------------------
if mer == 'Y' && GAP_T == 0;
    cd('./KS_OK');  como=ext;  files=dir(como);
    file1=files(1).name;   outfile=['./Merged/M' file1]; 
    Temporal=['./Merged/Temp' file1]; MergeSegy(ext,Temporal);
    [data,HTR,HSGY] = ReadSegy(Temporal);
    for i=1:1:size(data,2),  HTR(i).TraceSequenceLine = i;  end;
    WriteSegyStructure(outfile,HSGY,HTR,data,'revision',REV_Out,'dsf',DSF_Out);
    delete(Temporal); disp 'Merged in'; disp (outfile);
end;
%--------------------------------------------------
 disp ' ';disp 'SEG4KING END';fclose all;beep;


